@@ -6,8 +6,8 @@ see the C{L{openid.consumer.consumer}} module.  For information on
 implementing an OpenID server, see the C{L{openid.server.server}}
 module.
 
-@contact: U{dev@lists.openidenabled.com
-    <http://lists.openidenabled.com/mailman/listinfo/dev>}
+@contact: U{http://openid.net/developers/dev-mailing-lists/
+    <http://openid.net/developers/dev-mailing-lists/}
 
 @copyright: (C) 2005-2008 JanRain, Inc.
 
@@ -23,7 +23,7 @@ module.
     and limitations under the License.
 """
 
-__version__ = '[library version:2.2.1]'[17:-1]
+__version__ = '2.3.0'
 
 __all__ = [
     'association',
@@ -41,15 +41,4 @@ __all__ = [
     'store',
     'urinorm',
     'yadis',
-    ]
-
-# Parse the version info
-try:
-    version_info = map(int, __version__.split('.'))
-except ValueError:
-    version_info = (None, None, None)
-else:
-    if len(version_info) != 3:
-        version_info = (None, None, None)
-    else:
-        version_info = tuple(version_info)
+]
