@@ -11,21 +11,20 @@ __all__ = [
     'OPENID_IDP_2_0_TYPE',
     'OpenIDServiceEndpoint',
     'discover',
-    ]
+]
 
+import logging
 import urlparse
 
-from openid import oidutil, fetchers, urinorm
-
-from openid import yadis
-from openid.yadis.etxrd import nsTag, XRDSError, XRD_NS_2_0
-from openid.yadis.services import applyFilter as extractServices
-from openid.yadis.discover import discover as yadisDiscover
-from openid.yadis.discover import DiscoveryFailure
-from openid.yadis import xrires, filters
-from openid.yadis import xri
-
+from openid import fetchers, urinorm
 from openid.consumer import html_parse
+from openid.message import OPENID1_NS as OPENID_1_0_MESSAGE_NS, OPENID2_NS as OPENID_2_0_MESSAGE_NS
+from openid.yadis import filters, xri, xrires
+from openid.yadis.discover import DiscoveryFailure, discover as yadisDiscover
+from openid.yadis.etxrd import XRD_NS_2_0, XRDSError, nsTag
+from openid.yadis.services import applyFilter as extractServices
+
+_LOGGER = logging.getLogger(__name__)
 
 OPENID_1_0_NS = 'http://openid.net/xmlns/1.0'
 OPENID_IDP_2_0_TYPE = 'http://specs.openid.net/auth/2.0/server'
@@ -33,8 +32,6 @@ OPENID_2_0_TYPE = 'http://specs.openid.net/auth/2.0/signon'
 OPENID_1_1_TYPE = 'http://openid.net/signon/1.1'
 OPENID_1_0_TYPE = 'http://openid.net/signon/1.0'
 
-from openid.message import OPENID1_NS as OPENID_1_0_MESSAGE_NS
-from openid.message import OPENID2_NS as OPENID_2_0_MESSAGE_NS
 
 class OpenIDServiceEndpoint(object):
     """Object representing an OpenID service endpoint.
@@ -51,7 +48,7 @@ class OpenIDServiceEndpoint(object):
         OPENID_2_0_TYPE,
         OPENID_1_1_TYPE,
         OPENID_1_0_TYPE,
-        ]
+    ]
 
     def __init__(self):
         self.claimed_id = None
@@ -59,15 +56,14 @@ class OpenIDServiceEndpoint(object):
         self.type_uris = []
         self.local_id = None
         self.canonicalID = None
-        self.used_yadis = False # whether this came from an XRDS
+        self.used_yadis = False  # whether this came from an XRDS
         self.display_identifier = None
 
     def usesExtension(self, extension_uri):
         return extension_uri in self.type_uris
 
     def preferredNamespace(self):
-        if (OPENID_IDP_2_0_TYPE in self.type_uris or
-            OPENID_2_0_TYPE in self.type_uris):
+        if (OPENID_IDP_2_0_TYPE in self.type_uris or OPENID_2_0_TYPE in self.type_uris):
             return OPENID_2_0_MESSAGE_NS
         else:
             return OPENID_1_0_MESSAGE_NS
@@ -77,10 +73,7 @@ class OpenIDServiceEndpoint(object):
 
         I consider C{/server} endpoints to implicitly support C{/signon}.
         """
-        return (
-            (type_uri in self.type_uris) or 
-            (type_uri == OPENID_2_0_TYPE and self.isOPIdentifier())
-            )
+        return ((type_uri in self.type_uris) or (type_uri == OPENID_2_0_TYPE and self.isOPIdentifier()))
 
     def getDisplayIdentifier(self):
         """Return the display_identifier if set, else return the claimed_id.
@@ -126,6 +119,7 @@ class OpenIDServiceEndpoint(object):
         else:
             return self.local_id or self.canonicalID
 
+    @classmethod
     def fromBasicServiceEndpoint(cls, endpoint):
         """Create a new instance of this class from the endpoint
         object passed in.
@@ -147,8 +141,7 @@ class OpenIDServiceEndpoint(object):
 
         return openid_endpoint
 
-    fromBasicServiceEndpoint = classmethod(fromBasicServiceEndpoint)
-
+    @classmethod
     def fromHTML(cls, uri, html):
         """Parse the given document as HTML looking for an OpenID <link
         rel=...>
@@ -158,7 +151,7 @@ class OpenIDServiceEndpoint(object):
         discovery_types = [
             (OPENID_2_0_TYPE, 'openid2.provider', 'openid2.local_id'),
             (OPENID_1_1_TYPE, 'openid.server', 'openid.delegate'),
-            ]
+        ]
 
         link_attrs = html_parse.parseLinkAttrs(html)
         services = []
@@ -179,9 +172,7 @@ class OpenIDServiceEndpoint(object):
 
         return services
 
-    fromHTML = classmethod(fromHTML)
-
-
+    @classmethod
     def fromXRDS(cls, uri, xrds):
         """Parse the given document as XRDS looking for OpenID services.
 
@@ -193,9 +184,7 @@ class OpenIDServiceEndpoint(object):
         """
         return extractServices(uri, xrds, cls)
 
-    fromXRDS = classmethod(fromXRDS)
-
-
+    @classmethod
     def fromDiscoveryResult(cls, discoveryResult):
         """Create endpoints from a DiscoveryResult.
 
@@ -214,9 +203,7 @@ class OpenIDServiceEndpoint(object):
         return method(discoveryResult.normalized_uri,
                       discoveryResult.response_text)
 
-    fromDiscoveryResult = classmethod(fromDiscoveryResult)
-
-
+    @classmethod
     def fromOPEndpointURL(cls, op_endpoint_url):
         """Construct an OP-Identifier OpenIDServiceEndpoint object for
         a given OP Endpoint URL
@@ -229,9 +216,6 @@ class OpenIDServiceEndpoint(object):
         service.type_uris = [OPENID_IDP_2_0_TYPE]
         return service
 
-    fromOPEndpointURL = classmethod(fromOPEndpointURL)
-
-
     def __str__(self):
         return ("<%s.%s "
                 "server_url=%r "
@@ -240,7 +224,7 @@ class OpenIDServiceEndpoint(object):
                 "canonicalID=%r "
                 "used_yadis=%s "
                 ">"
-                 % (self.__class__.__module__, self.__class__.__name__,
+                % (self.__class__.__module__, self.__class__.__name__,
                     self.server_url,
                     self.claimed_id,
                     self.local_id,
@@ -248,7 +232,6 @@ class OpenIDServiceEndpoint(object):
                     self.used_yadis))
 
 
-
 def findOPLocalIdentifier(service_element, type_uris):
     """Find the OP-Local Identifier for this xrd:Service element.
 
@@ -278,8 +261,7 @@ def findOPLocalIdentifier(service_element, type_uris):
 
     # Build the list of tags that could contain the OP-Local Identifier
     local_id_tags = []
-    if (OPENID_1_1_TYPE in type_uris or
-        OPENID_1_0_TYPE in type_uris):
+    if (OPENID_1_1_TYPE in type_uris or OPENID_1_0_TYPE in type_uris):
         local_id_tags.append(nsTag(OPENID_1_0_NS, 'Delegate'))
 
     if OPENID_2_0_TYPE in type_uris:
@@ -299,27 +281,29 @@ def findOPLocalIdentifier(service_element, type_uris):
 
     return local_id
 
+
 def normalizeURL(url):
     """Normalize a URL, converting normalization failures to
     DiscoveryFailure"""
     try:
         normalized = urinorm.urinorm(url)
-    except ValueError, why:
+    except ValueError as why:
         raise DiscoveryFailure('Normalizing identifier: %s' % (why[0],), None)
     else:
         return urlparse.urldefrag(normalized)[0]
 
+
+def normalizeXRI(xri):
+    """Normalize an XRI, stripping its scheme if present"""
+    if xri.startswith("xri://"):
+        xri = xri[6:]
+    return xri
+
+
 def arrangeByType(service_list, preferred_types):
     """Rearrange service_list in a new list so services are ordered by
     types listed in preferred_types.  Return the new list."""
 
-    def enumerate(elts):
-        """Return an iterable that pairs the index of an element with
-        that element.
-
-        For Python 2.2 compatibility"""
-        return zip(range(len(elts)), elts)
-
     def bestMatchingService(service):
         """Return the index of the first matching type, or something
         higher if no type matches.
@@ -337,9 +321,7 @@ def arrangeByType(service_list, preferred_types):
 
     # Build a list with the service elements in tuples whose
     # comparison will prefer the one with the best matching service
-    prio_services = [(bestMatchingService(s), orig_index, s)
-                     for (orig_index, s) in enumerate(service_list)]
-    prio_services.sort()
+    prio_services = sorted((bestMatchingService(s), orig_index, s) for (orig_index, s) in enumerate(service_list))
 
     # Now that the services are sorted by priority, remove the sort
     # keys from the list.
@@ -348,6 +330,7 @@ def arrangeByType(service_list, preferred_types):
 
     return prio_services
 
+
 def getOPOrUserServices(openid_services):
     """Extract OP Identifier services.  If none found, return the
     rest, sorted with most preferred first according to
@@ -364,6 +347,7 @@ def getOPOrUserServices(openid_services):
 
     return op_services or openid_services
 
+
 def discoverYadis(uri):
     """Discover OpenID services for a URI. Tries Yadis and falls back
     on old-style <link rel='...'> discovery if Yadis fails.
@@ -405,8 +389,10 @@ def discoverYadis(uri):
 
     return (yadis_url, getOPOrUserServices(openid_services))
 
+
 def discoverXRI(iname):
     endpoints = []
+    iname = normalizeXRI(iname)
     try:
         canonicalID, services = xrires.ProxyResolver().query(
             iname, OpenIDServiceEndpoint.openid_type_uris)
@@ -418,7 +404,7 @@ def discoverXRI(iname):
         for service_element in services:
             endpoints.extend(flt.getServiceEndpoints(iname, service_element))
     except XRDSError:
-        oidutil.log('xrds error on ' + iname)
+        _LOGGER.exception('xrds error on %s', iname)
 
     for endpoint in endpoints:
         # Is there a way to pass this through the filter to the endpoint
@@ -443,6 +429,7 @@ def discoverNoYadis(uri):
         claimed_id, http_resp.body)
     return claimed_id, openid_services
 
+
 def discoverURI(uri):
     parsed = urlparse.urlparse(uri)
     if parsed[0] and parsed[1]:
@@ -456,6 +443,7 @@ def discoverURI(uri):
     claimed_id = normalizeURL(claimed_id)
     return claimed_id, openid_services
 
+
 def discover(identifier):
     if xri.identifierScheme(identifier) == "XRI":
         return discoverXRI(identifier)
