@@ -4,14 +4,16 @@ various SQL databases to back them.
 
 Example of how to initialize a store database::
 
-    python -c 'from openid.store import sqlstore; import pysqlite2.dbapi2; sqlstore.SQLiteStore(pysqlite2.dbapi2.connect("cstore.db")).createTables()'
+    python -c 'from openid.store import sqlstore; import pysqlite2.dbapi2;
+               sqlstore.SQLiteStore(pysqlite2.dbapi2.connect("cstore.db")).createTables()'
 """
 import re
 import time
 
 from openid.association import Association
-from openid.store.interface import OpenIDStore
 from openid.store import nonce
+from openid.store.interface import OpenIDStore
+
 
 def _inTxn(func):
     def wrapped(self, *args, **kwargs):
@@ -28,13 +30,14 @@ def _inTxn(func):
 
     return wrapped
 
+
 class SQLStore(OpenIDStore):
     """
     This is the parent class for the SQL stores, which contains the
     logic common to all of the SQL stores.
 
     The table names used are determined by the class variables
-    C{L{settings_table}}, C{L{associations_table}}, and
+    C{L{associations_table}} and
     C{L{nonces_table}}.  To change the name of the tables used, pass
     new table names into the constructor.
 
@@ -49,9 +52,6 @@ class SQLStore(OpenIDStore):
     should be considered implementation details.
 
 
-    @cvar settings_table: This is the default name of the table to
-        keep this store's settings in.
-
     @cvar associations_table: This is the default name of the table to
         keep associations in
 
@@ -62,12 +62,10 @@ class SQLStore(OpenIDStore):
     @sort: __init__, createTables
     """
 
-    settings_table = 'oid_settings'
     associations_table = 'oid_associations'
     nonces_table = 'oid_nonces'
 
-    def __init__(self, conn, settings_table=None, associations_table=None,
-                 nonces_table=None):
+    def __init__(self, conn, associations_table=None, nonces_table=None):
         """
         This creates a new SQLStore instance.  It requires an
         established database connection be given to it, and it allows
@@ -82,14 +80,6 @@ class SQLStore(OpenIDStore):
             object.
 
 
-        @param settings_table: This is an optional parameter to
-            specify the name of the table used for this store's
-            settings.  The default value is specified in
-            C{L{SQLStore.settings_table}}.
-
-        @type settings_table: C{str}
-
-
         @param associations_table: This is an optional parameter to
             specify the name of the table used for storing
             associations.  The default value is specified in
@@ -108,11 +98,21 @@ class SQLStore(OpenIDStore):
         self.cur = None
         self._statement_cache = {}
         self._table_names = {
-            'settings': settings_table or self.settings_table,
             'associations': associations_table or self.associations_table,
             'nonces': nonces_table or self.nonces_table,
-            }
-        self.max_nonce_age = 6 * 60 * 60 # Six hours, in seconds
+        }
+        self.max_nonce_age = 6 * 60 * 60  # Six hours, in seconds
+
+        # DB API extension: search for "Connection Attributes .Error,
+        # .ProgrammingError, etc." in
+        # http://www.python.org/dev/peps/pep-0249/
+        if hasattr(self.conn, 'IntegrityError') and hasattr(self.conn, 'OperationalError'):
+            self.exceptions = self.conn
+
+        if not (hasattr(self.exceptions, 'IntegrityError') and
+                hasattr(self.exceptions, 'OperationalError')):
+            raise RuntimeError("Error using database connection module "
+                               "(Maybe it can't be imported?)")
 
     def blobDecode(self, blob):
         """Convert a blob as returned by the SQL engine into a str object.
@@ -136,7 +136,18 @@ class SQLStore(OpenIDStore):
 
     def _execSQL(self, sql_name, *args):
         sql = self._getSQL(sql_name)
-        self.cur.execute(sql, args)
+        # Kludge because we have reports of postgresql not quoting
+        # arguments if they are passed in as unicode instead of str.
+        # Currently the strings in our tables just have ascii in them,
+        # so this ought to be safe.
+
+        def unicode_to_str(arg):
+            if isinstance(arg, unicode):
+                return str(arg)
+            else:
+                return arg
+        str_args = [unicode_to_str(i) for i in args]
+        self.cur.execute(sql, str_args)
 
     def __getattr__(self, attr):
         # if the attribute starts with db_, use a default
@@ -144,6 +155,7 @@ class SQLStore(OpenIDStore):
         # as an attribute of this object and executes it.
         if attr[:3] == 'db_':
             sql_name = attr[3:] + '_sql'
+
             def func(*args):
                 return self._execSQL(sql_name, *args)
             setattr(self, attr, func)
@@ -165,7 +177,7 @@ class SQLStore(OpenIDStore):
             finally:
                 self.cur.close()
                 self.cur = None
-        except:
+        except Exception:
             self.conn.rollback()
             raise
         else:
@@ -181,7 +193,6 @@ class SQLStore(OpenIDStore):
         """
         self.db_create_nonce()
         self.db_create_assoc()
-        self.db_create_settings()
 
     createTables = _inTxn(txn_createTables)
 
@@ -240,7 +251,7 @@ class SQLStore(OpenIDStore):
         (str, str) -> bool
         """
         self.db_remove_assoc(server_url, handle)
-        return self.cur.rowcount > 0 # -1 is undefined
+        return self.cur.rowcount > 0  # -1 is undefined
 
     removeAssociation = _inTxn(txn_removeAssociation)
 
@@ -254,7 +265,7 @@ class SQLStore(OpenIDStore):
 
         try:
             self.db_add_nonce(server_url, timestamp, salt)
-        except self.dbapi.IntegrityError:
+        except self.exceptions.IntegrityError:
             # The key uniqueness check failed
             return False
         else:
@@ -286,11 +297,6 @@ class SQLiteStore(SQLStore):
     All other methods are implementation details.
     """
 
-    try:
-        from pysqlite2 import dbapi2 as dbapi
-    except ImportError:
-        pass
-
     create_nonce_sql = """
     CREATE TABLE %(nonces)s (
         server_url VARCHAR,
@@ -313,15 +319,9 @@ class SQLiteStore(SQLStore):
     );
     """
 
-    create_settings_sql = """
-    CREATE TABLE %(settings)s
-    (
-        setting VARCHAR(128) UNIQUE PRIMARY KEY,
-        value BLOB(20)
-    );
-    """
-
     set_assoc_sql = ('INSERT OR REPLACE INTO %(associations)s '
+                     '(server_url, handle, secret, issued, '
+                     'lifetime, assoc_type) '
                      'VALUES (?, ?, ?, ?, ?, ?);')
     get_assocs_sql = ('SELECT handle, secret, issued, lifetime, assoc_type '
                       'FROM %(associations)s WHERE server_url = ?;')
@@ -353,12 +353,13 @@ class SQLiteStore(SQLStore):
         # message from the OperationalError.
         try:
             return super(SQLiteStore, self).useNonce(*args, **kwargs)
-        except self.dbapi.OperationalError, why:
+        except self.exceptions.OperationalError as why:
             if re.match('^columns .* are not unique$', why[0]):
                 return False
             else:
                 raise
 
+
 class MySQLStore(SQLStore):
     """
     This is a MySQL-based specialization of C{L{SQLStore}}.
@@ -372,41 +373,32 @@ class MySQLStore(SQLStore):
     """
 
     try:
-        import MySQLdb as dbapi
+        import MySQLdb as exceptions
     except ImportError:
-        pass
+        exceptions = None
 
     create_nonce_sql = """
     CREATE TABLE %(nonces)s (
-        server_url BLOB,
-        timestamp INTEGER,
-        salt CHAR(40),
+        server_url BLOB NOT NULL,
+        timestamp INTEGER NOT NULL,
+        salt CHAR(40) NOT NULL,
         PRIMARY KEY (server_url(255), timestamp, salt)
     )
-    TYPE=InnoDB;
+    ENGINE=InnoDB;
     """
 
     create_assoc_sql = """
     CREATE TABLE %(associations)s
     (
-        server_url BLOB,
-        handle VARCHAR(255),
-        secret BLOB,
-        issued INTEGER,
-        lifetime INTEGER,
-        assoc_type VARCHAR(64),
+        server_url BLOB NOT NULL,
+        handle VARCHAR(255) NOT NULL,
+        secret BLOB NOT NULL,
+        issued INTEGER NOT NULL,
+        lifetime INTEGER NOT NULL,
+        assoc_type VARCHAR(64) NOT NULL,
         PRIMARY KEY (server_url(255), handle)
     )
-    TYPE=InnoDB;
-    """
-
-    create_settings_sql = """
-    CREATE TABLE %(settings)s
-    (
-        setting VARCHAR(128) UNIQUE PRIMARY KEY,
-        value BLOB
-    )
-    TYPE=InnoDB;
+    ENGINE=InnoDB;
     """
 
     set_assoc_sql = ('REPLACE INTO %(associations)s '
@@ -429,7 +421,13 @@ class MySQLStore(SQLStore):
     clean_nonce_sql = 'DELETE FROM %(nonces)s WHERE timestamp < %%s;'
 
     def blobDecode(self, blob):
-        return blob.tostring()
+        if isinstance(blob, str):
+            # Versions of MySQLdb >= 1.2.2
+            return blob
+        else:
+            # Versions of MySQLdb prior to 1.2.2 (as far as we can tell)
+            return blob.tostring()
+
 
 class PostgreSQLStore(SQLStore):
     """
@@ -442,15 +440,20 @@ class PostgreSQLStore(SQLStore):
     """
 
     try:
-        import psycopg as dbapi
+        import psycopg as exceptions
     except ImportError:
-        pass
+        # psycopg2 has the dbapi extension where the exception classes
+        # are available on the connection object. A psycopg2
+        # connection will use the correct exception classes because of
+        # this, and a psycopg connection will fall through to use the
+        # psycopg imported above.
+        exceptions = None
 
     create_nonce_sql = """
     CREATE TABLE %(nonces)s (
-        server_url VARCHAR(2047),
-        timestamp INTEGER,
-        salt CHAR(40),
+        server_url VARCHAR(2047) NOT NULL,
+        timestamp INTEGER NOT NULL,
+        salt CHAR(40) NOT NULL,
         PRIMARY KEY (server_url, timestamp, salt)
     );
     """
@@ -458,33 +461,24 @@ class PostgreSQLStore(SQLStore):
     create_assoc_sql = """
     CREATE TABLE %(associations)s
     (
-        server_url VARCHAR(2047),
-        handle VARCHAR(255),
-        secret BYTEA,
-        issued INTEGER,
-        lifetime INTEGER,
-        assoc_type VARCHAR(64),
+        server_url VARCHAR(2047) NOT NULL,
+        handle VARCHAR(255) NOT NULL,
+        secret BYTEA NOT NULL,
+        issued INTEGER NOT NULL,
+        lifetime INTEGER NOT NULL,
+        assoc_type VARCHAR(64) NOT NULL,
         PRIMARY KEY (server_url, handle),
         CONSTRAINT secret_length_constraint CHECK (LENGTH(secret) <= 128)
     );
     """
 
-    create_settings_sql = """
-    CREATE TABLE %(settings)s
-    (
-        setting VARCHAR(128) UNIQUE PRIMARY KEY,
-        value BYTEA,
-        CONSTRAINT value_length_constraint CHECK (LENGTH(value) <= 20)
-    );
-    """
-
     def db_set_assoc(self, server_url, handle, secret, issued, lifetime, assoc_type):
         """
         Set an association.  This is implemented as a method because
         REPLACE INTO is not supported by PostgreSQL (and is not
         standard SQL).
         """
-        result = self.db_get_assoc(server_url, handle)
+        self.db_get_assoc(server_url, handle)
         rows = self.cur.fetchall()
         if len(rows):
             # Update the table since this associations already exists.
@@ -520,5 +514,9 @@ class PostgreSQLStore(SQLStore):
     clean_nonce_sql = 'DELETE FROM %(nonces)s WHERE timestamp < %%s;'
 
     def blobEncode(self, blob):
-        import psycopg
-        return psycopg.Binary(blob)
+        try:
+            from psycopg2 import Binary
+        except ImportError:
+            from psycopg import Binary
+
+        return Binary(blob)
