@@ -1,14 +1,13 @@
 # -*- test-case-name: openid.test.test_yadis_discover -*-
 __all__ = ['discover', 'DiscoveryResult', 'DiscoveryFailure']
 
-from cStringIO import StringIO
+from StringIO import StringIO
 
 from openid import fetchers
-
-from openid.yadis.constants import \
-     YADIS_HEADER_NAME, YADIS_CONTENT_TYPE, YADIS_ACCEPT_HEADER
+from openid.yadis.constants import YADIS_ACCEPT_HEADER, YADIS_CONTENT_TYPE, YADIS_HEADER_NAME
 from openid.yadis.parsehtml import MetaNotFound, findHTMLMeta
 
+
 class DiscoveryFailure(Exception):
     """Raised when a YADIS protocol error occurs in the discovery process"""
     identity_url = None
@@ -17,6 +16,7 @@ class DiscoveryFailure(Exception):
         Exception.__init__(self, message)
         self.http_response = http_response
 
+
 class DiscoveryResult(object):
     """Contains the result of performing Yadis discovery on a URI"""
 
@@ -45,6 +45,8 @@ class DiscoveryResult(object):
 
     def usedYadisLocation(self):
         """Was the Yadis protocol's indirection used?"""
+        if self.xrds_uri is None:
+            return False
         return self.normalized_uri != self.xrds_uri
 
     def isXRDS(self):
@@ -52,6 +54,7 @@ class DiscoveryResult(object):
         return (self.usedYadisLocation() or
                 self.content_type == YADIS_CONTENT_TYPE)
 
+
 def discover(uri):
     """Discover services for a given URI.
 
@@ -68,7 +71,7 @@ def discover(uri):
     """
     result = DiscoveryResult(uri)
     resp = fetchers.fetch(uri, headers={'Accept': YADIS_ACCEPT_HEADER})
-    if resp.status != 200:
+    if resp.status not in (200, 206):
         raise DiscoveryFailure(
             'HTTP Response status from identity URL host is not 200. '
             'Got status %r' % (resp.status,), resp)
@@ -84,7 +87,7 @@ def discover(uri):
 
     if result.xrds_uri and result.usedYadisLocation():
         resp = fetchers.fetch(result.xrds_uri)
-        if resp.status != 200:
+        if resp.status not in (200, 206):
             exc = DiscoveryFailure(
                 'HTTP Response status from Yadis host is not 200. '
                 'Got status %r' % (resp.status,), resp)
@@ -96,7 +99,6 @@ def discover(uri):
     return result
 
 
-
 def whereIsYadis(resp):
     """Given a HTTPResponse, return the location of the Yadis document.
 
@@ -113,8 +115,7 @@ def whereIsYadis(resp):
 
     # According to the spec, the content-type header must be an exact
     # match, or else we have to look for an indirection.
-    if (content_type and
-        content_type.split(';', 1)[0].lower() == YADIS_CONTENT_TYPE):
+    if content_type and content_type.split(';', 1)[0].lower() == YADIS_CONTENT_TYPE:
         return resp.final_url
     else:
         # Try the header
@@ -126,10 +127,27 @@ def whereIsYadis(resp):
             # XXX: do we want to do something with content-type, like
             # have a whitelist or a blacklist (for detecting that it's
             # HTML)?
+
+            # Decode body by encoding of file
+            content_type = content_type or ''
+            encoding = content_type.rsplit(';', 1)
+            if len(encoding) == 2 and encoding[1].strip().startswith('charset='):
+                encoding = encoding[1].split('=', 1)[1].strip()
+            else:
+                encoding = 'UTF-8'
+
             try:
-                yadis_loc = findHTMLMeta(StringIO(resp.body))
-            except MetaNotFound:
+                content = resp.body.decode(encoding)
+            except UnicodeError:
+                # Keep encoded version in case yadis location can be found before encoding shut this up.
+                # Possible errors will be caught lower.
+                content = resp.body
+
+            try:
+                yadis_loc = findHTMLMeta(StringIO(content))
+            except (MetaNotFound, UnicodeError):
+                # UnicodeError: Response body could not be encoded and xrds location
+                # could not be found before troubles occurs.
                 pass
 
         return yadis_loc
-
